package com.wechat.service.pay.impl;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.UUID;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.wechat.common.entity.order.KtjJsapiOrder;
import com.wechat.common.entity.order.KtjRefundOrder;
import com.wechat.common.entity.participation.RefundParticipation;
import com.wechat.common.entity.pay.jsapi.JsapiUnifiedOrder;
import com.wechat.common.enums.RefundStatus;
import com.wechat.common.parameter.ProjectConstant;
import com.wechat.common.response.RespCode;
import com.wechat.common.response.RespWrapper;
import com.wechat.common.util.ClientCustomSSL;
import com.wechat.common.util.StringUtil;
import com.wechat.common.util.WeChatUtil;
import com.wechat.dao.order.KtjJsapiOrderDao;
import com.wechat.dao.order.KtjRefundOrderDao;
import com.wechat.dao.pay.WeChatPayJsapiDao;
import com.wechat.dao.sepAccount.SepRecordDao;
import com.wechat.entity.sepAccount.KtjSepRecord;
import com.wechat.service.pay.WeChatPayJsapiService;

@Service
public class WeChatPayJsapiServiceImpl implements WeChatPayJsapiService{
	
	@Autowired
	private WeChatPayJsapiDao weChatPayJsapiDao;
	
	@Autowired
	private KtjJsapiOrderDao ktjJsapiOrderDao;
	
	@Autowired
	private KtjRefundOrderDao ktjRefundOrderDao;
	
	@Autowired
	private SepRecordDao sepRecordDao;
	
	@Override
	public RespWrapper<Object> jsapiUnifiedOrder(JsapiUnifiedOrder unifiedOrder) {
		try {
			//参数验证
			RespWrapper<Object> flag = unifiedOrder.parameterValidation();
			if (RespCode.SUCCESS != flag.getCode()) {
				return flag;
			}
			//查找商户所配置的小程序
			Map<String,Object> map = weChatPayJsapiDao.queryAppId(unifiedOrder);
			if(null == map || null == map.get("id")){
				return RespWrapper.makeFailResp("特邀商户号不存在，请检查！");
			}
			//生成微信请求数据
			Map<String,Object> data = createMap(map,unifiedOrder);
			String postXmlString = WeChatUtil.mapFormatXML(data);
			String xmlStr = WeChatUtil.postWX(ProjectConstant.jsapiUrl, postXmlString);
			//将微信返回的xml转成map集合
			Map<String, Object> httpResp = WeChatUtil.parseXML(xmlStr);
			//存储订单数据
			KtjJsapiOrder order = new KtjJsapiOrder();
			order.assignment(unifiedOrder,data.get("out_trade_no"),map.get("miniProgramSubAppid"));
			ktjJsapiOrderDao.insert(order);
			//返回微信支付数据
			return returnMap(httpResp,map);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器错误，请稍后重试！");
		}
		
	}
	
	/**
	 * 生成微信支付数据
	 * @param httpResp
	 * @param map
	 * @return
	 */
	private RespWrapper<Object> returnMap(Map<String, Object> httpResp,Map<String, Object> map) {
		Map<String, Object> respMap = new HashMap<>();
		respMap.put("appId", map.get("miniProgramSubAppid"));
        respMap.put("timeStamp", String.valueOf(System.currentTimeMillis() / 1000)); // 当前的时间戳,秒
        respMap.put("nonceStr", UUID.randomUUID().toString().replaceAll("-", "").toUpperCase()); // 随机字符串
        respMap.put("package", "prepay_id=" + httpResp.get("prepay_id")); // 微信统一下单接口返回的prepay_id参数值，提交格式如：prepay_id=wx112738712639781263128736
        respMap.put("signType", "MD5"); // 签名算法,暂时只支持MD5
        respMap.put("paySign", WeChatUtil.createSign(respMap, ProjectConstant.api)); // 签名
		return RespWrapper.makeSuccessResp(respMap);
	}

	private Map<String, Object> createMap(Map<String, Object> map,JsapiUnifiedOrder unifiedOrder) {
		String nonceStr =  UUID.randomUUID().toString().replaceAll("-", "").toUpperCase();
		String orderNo = "Comtika_"+ System.currentTimeMillis() / 1000 + "_"+ System.currentTimeMillis() / 1000000;
		Map<String,Object> data = new HashMap<String, Object>();
		data.put("appid", ProjectConstant.appid);
		data.put("mch_id", ProjectConstant.merchantId);
		data.put("sub_appid", map.get("miniProgramSubAppid"));
		data.put("sub_mch_id", unifiedOrder.getSubMchid());
		data.put("nonce_str", nonceStr);
		data.put("body", unifiedOrder.getBody());
		data.put("out_trade_no", orderNo);
		data.put("profit_sharing", "Y");//设定订单是可以分账的订单
		data.put("total_fee", unifiedOrder.getTotalFee());
		data.put("spbill_create_ip", unifiedOrder.getSpbillCreateIp());
		data.put("notify_url", ProjectConstant.notifyUrl);
		data.put("sub_openid", unifiedOrder.getSubOpenid());
		data.put("trade_type", "JSAPI");
		data.put("sign", WeChatUtil.createSign(data, ProjectConstant.api));
		return data;
	}

	@Override
	public void notifyOrder(HttpServletRequest request,HttpServletResponse response) {
		BufferedReader br = null;
		try {
			br = new BufferedReader(new InputStreamReader(request.getInputStream(), "UTF-8"));
 	        StringBuffer buffer = new StringBuffer();
 	        String line = "";
 	        while ((line = br.readLine()) != null) {
 	            buffer.append(line);
 	        }
 	        System.out.println("微信返回信息:" + buffer.toString());
 	        Map<String, Object> httpResp = WeChatUtil.parseXML(buffer.toString());
 	        String orderNo = String.valueOf(httpResp.get("out_trade_no"));//微信外部单号，就是本系统订单号
	        String transactionId = String.valueOf(httpResp.get("transaction_id"));//微信支付单号
 	        KtjJsapiOrder order = new KtjJsapiOrder();
 	        order.setOrderNo(orderNo);
 	        order = ktjJsapiOrderDao.templateOne(order);
 	        if(null != order){
 	        	//通知微信
 	        	WeChatUtil.write2wx(response,"SUCCESS","OK");
 	        	//修改数据库
 	        	KtjJsapiOrder order1 = new KtjJsapiOrder();
 	        	order1.setId(order.getId());
 	        	order1.setTransactionId(transactionId);
 	        	order1.setOrderStatus(1);
 	        	ktjJsapiOrderDao.updateTemplateById(order1);
 	        	//回调第三方
 	        	////....此处先忽略
 	        }
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public static void main(String[] args) {
		/*String aa = "<xml><appid><![CDATA[wxec504e0378010e16]]></appid><bank_type><![CDATA[OTHERS]]></bank_type><cash_fee><![CDATA[2]]></cash_fee><fee_type><![CDATA[CNY]]></fee_type><is_subscribe><![CDATA[Y]]></is_subscribe><mch_id><![CDATA[1503800091]]></mch_id><nonce_str><![CDATA[D692E9AF473D43E7A74160E841ABD7AF]]></nonce_str><openid><![CDATA[oJ-kq1JgWwDyeRK3BJ7rkL1AenhI]]></openid><out_trade_no><![CDATA[Comtika_1586422175]]></out_trade_no><result_code><![CDATA[SUCCESS]]></result_code><return_code><![CDATA[SUCCESS]]></return_code><sign><![CDATA[82B5174BF3A133915DEEEE86EA2101AB]]></sign><sub_appid><![CDATA[wx9b1d7edef2f57d51]]></sub_appid><sub_is_subscribe><![CDATA[N]]></sub_is_subscribe><sub_mch_id><![CDATA[1584278251]]></sub_mch_id><sub_openid><![CDATA[odp725dgFF0NLLOlL6YtvVtMiEW8]]></sub_openid><time_end><![CDATA[20200409165252]]></time_end><total_fee>2</total_fee><trade_type><![CDATA[JSAPI]]></trade_type><transaction_id><![CDATA[4200000500202004098650055482]]></transaction_id></xml>";
		Map<String, Object> httpResp = WeChatUtil.parseXML(aa);
		for(Entry<String, Object> str : httpResp.entrySet()){
			System.out.println(str.getKey() + ":" + str.getValue());
		}*/
		/*String time = "2016-07-25 15:26:26";
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		try {
			System.out.println(sdf.parse(time).getTime());
		} catch (ParseException e) {
			e.printStackTrace();
		}*/
		String abc = "<xml><return_code><![CDATA[SUCCESS]]></return_code><return_msg><![CDATA[OK]]></return_msg><appid><![CDATA[wxec504e0378010e16]]></appid><mch_id><![CDATA[1503800091]]></mch_id><sub_mch_id><![CDATA[1584278251]]></sub_mch_id><nonce_str><![CDATA[FNl1cqSrmYQYkpHB]]></nonce_str><sign><![CDATA[2E2474193D8A75CA1BD667261473B58F]]></sign><result_code><![CDATA[SUCCESS]]></result_code><transaction_id><![CDATA[4200000529202004182527501049]]></transaction_id><out_trade_no><![CDATA[Comtika_1587178537_1587178]]></out_trade_no><out_refund_no><![CDATA[Comtika_Refund_15872181986701587218]]></out_refund_no><refund_id><![CDATA[50300104082020041800169870512]]></refund_id><refund_channel><![CDATA[]]></refund_channel><refund_fee>4</refund_fee><coupon_refund_fee>0</coupon_refund_fee><total_fee>4</total_fee><cash_fee>4</cash_fee><coupon_refund_count>0</coupon_refund_count><cash_refund_fee>4</cash_refund_fee></xml>";
		Map<String, Object> httpResp = WeChatUtil.parseXML(abc);
		String returnCode = String.valueOf(httpResp.get("return_code"));
		String resultCode = String.valueOf(httpResp.get("result_code"));
		if("SUCCESS".equals(returnCode)){
			if("SUCCESS".equals(resultCode)){
				System.out.println(0);
			}else{
				System.out.println(1);
			}
		}
	}

	@Override
	public RespWrapper<Object> refundByOutOrderNo(RefundParticipation params) {
		try {
			KtjJsapiOrder order = new KtjJsapiOrder();
			RespWrapper<Object> data = params.paramsInfo(order);
			if( data.getCode() != RespCode.SUCCESS){
				return data;
			}
			order = ktjJsapiOrderDao.templateOne(order);
			if( null == order){
				return RespWrapper.makeFailResp("订单号不存在，请检查！");
			}
			if(order.getOrderStatus() != 1 ){
				return RespWrapper.makeFailResp("订单未支付，请勿发起退款！");
			}
			//检查退款单号是否已存在
			KtjRefundOrder reOrder = new KtjRefundOrder();
			reOrder.setOutRefundNo(params.getOutRefundNo());
			reOrder = ktjRefundOrderDao.templateOne(reOrder);
			if( null != reOrder){
				return RespWrapper.makeFailResp("退款单号已存在，请重新生成！");
			}
			//检查这笔订单是否已经发起了分账
			KtjSepRecord ktjSepRecord = new KtjSepRecord();
			ktjSepRecord.setTransactionId(params.getTransactionId());
			ktjSepRecord = sepRecordDao.templateOne(ktjSepRecord);
			if( null != ktjSepRecord){
				return RespWrapper.makeFailResp("该订单已经发起分账，请勿发起退款！");
			}
			//生成退款参数
			Map<String,Object> map = refundMap(order,params);
			String postXmlString = WeChatUtil.mapFormatXML(map);
			//使用双向证书调用微信
			String xmlStr = ClientCustomSSL.doCertWX(ProjectConstant.refundUrl, postXmlString);
System.out.println("微信返回结果："+xmlStr);
			//String xmlStr = WeChatUtil.postWX(ProjectConstant.refundUrl, postXmlString);
			//将微信返回的xml转成map集合
			Map<String, Object> httpResp = WeChatUtil.parseXML(xmlStr);
			String returnCode = String.valueOf(httpResp.get("return_code"));
			String resultCode = String.valueOf(httpResp.get("result_code"));
System.out.println("returnCode:"+returnCode+",resultCode:"+resultCode);
			if("SUCCESS".equals(returnCode)){
				if("SUCCESS".equals(resultCode)){
					String refundId = String.valueOf(httpResp.get("refund_id"));
					reOrder = new KtjRefundOrder();
					reOrder.setRefundId(refundId);
					reOrder.setRefundStatus(RefundStatus.REFUND_PROCESSING.getId());
					//存储退款数据
					reOrder.paramsOrder(map,params,order.getId());
					ktjRefundOrderDao.insertTemplate(reOrder, true);
					return RespWrapper.makeSuccessResp("退款请求成功！");
				}
 			}
			return RespWrapper.makeFailResp("退款请求失败，请重新发起退款！");
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器错误，请重新发起退款！");
		}
		
		
	}

	private Map<String, Object> refundMap(KtjJsapiOrder order,RefundParticipation params) {
		Map<String,Object> map = new HashMap<String, Object>();
		map.put("appid", ProjectConstant.appid);
		map.put("mch_id", ProjectConstant.merchantId);
		map.put("sub_appid", order.getAppid());
		map.put("sub_mch_id", order.getSubMchid());
		map.put("nonce_str", UUID.randomUUID().toString().replaceAll("-", "").toUpperCase());
		map.put("transaction_id", order.getTransactionId());
		map.put("out_refund_no", "Comtika_Refund_"+System.currentTimeMillis()+System.currentTimeMillis()/1000000);
		map.put("total_fee",order.getTotalFee());
		map.put("refund_fee", params.getRefundFee());
		map.put("refund_desc", params.getRefundDesc());
		map.put("notify_url", ProjectConstant.refundNotifyUrl);
		map.put("sign", WeChatUtil.createSign(map, ProjectConstant.api));
		return map;
	}

	@Override
	public void notifyRefund(HttpServletRequest request,HttpServletResponse response) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public RespWrapper<Object> queryRefundByOutOrderNo(String outRefundNo) {
		try {
			if(StringUtil.isEmpty(outRefundNo)){
				return RespWrapper.makeFailResp("退款订单编号不能为空！");
			}
			KtjRefundOrder reOrder = new KtjRefundOrder();
			reOrder.setOutRefundNo(outRefundNo);
			reOrder = ktjRefundOrderDao.templateOne(reOrder);
			if( null == reOrder){
				return RespWrapper.makeFailResp("退款订单编号不存在，请检查！");
			}
			Map<String,Object> map = new HashMap<String, Object>();
			map.put("refundStatus", reOrder.getRefundStatus());
			map.put("refundMessage", reOrder.getRefundMessage());
			return RespWrapper.makeSuccessResp(map);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器错误，请重新查询！");
		}
		
	}

	@Override
	public RespWrapper<Object> queryWechatRefundStatus(String refundId) {
		try {
			if(StringUtil.isEmpty(refundId)){
				return RespWrapper.makeFailResp("微信退款单号不能为空！");
			}
			KtjRefundOrder reOrder = new KtjRefundOrder();
			reOrder.setRefundId(refundId);
			Map<String,Object> data = ktjRefundOrderDao.queryRefundOrder(reOrder);
			if( null == data || null == data.get("subMchid")){
				return RespWrapper.makeFailResp("微信退款单号不存在，请检查！");
			}
			if(Integer.parseInt(String.valueOf(data.get("refundStatus")))  != RefundStatus.REFUND_PROCESSING.getId()){
				return RespWrapper.makeFailResp("此退款单不是退款中的订单，请勿发起查询！");
			}
			//上面sql已经查询出数据库id，需要赋值进去，然后下面可以修改退款订单的状态
			reOrder.setId(Long.parseLong(String.valueOf(data.get("id"))));
			Map<String,Object> map = new HashMap<String, Object>();
			//如果不是退款处理中的订单，就没有必要去查询微信
			if( reOrder.getRefundStatus() != RefundStatus.REFUND_PROCESSING.getId() ){
				map.put("refundStatus", reOrder.getRefundStatus());
				map.put("refundMessage", reOrder.getRefundMessage());
				return RespWrapper.makeSuccessResp(map);
			}
			//查询微信
			map =  queryRefundMap(data);
			String postXmlString = WeChatUtil.mapFormatXML(data);
			String xmlStr = WeChatUtil.postWX(ProjectConstant.jsapiUrl, postXmlString);
			//将微信返回的xml转成map集合
			Map<String, Object> httpResp = WeChatUtil.parseXML(xmlStr);
			Integer refundStatus;
			String status = "";
			Long refundSuccessTime = 0L;
			for (Entry<String, Object> entry : httpResp.entrySet()){
System.out.println(entry.getKey());
				String key = entry.getKey();
				//由于微信提供退款查询，但是退款状态的字段是不固定的refund_status_$n，$n为下标，从0开始编号。
				//所以只需要找key包含refund_status的就为微信退款状态
				if(key.contains("refund_status")){
					status = String.valueOf(entry.getValue());
					refundStatus = RefundStatus.getIdByType(status);
					reOrder.setRefundStatus(refundStatus);
				}
				//由于微信提供退款查询，但是退款状态的字段是不固定的refund_success_time_$n	，$n为下标，从0开始编号。
				//所以只需要找key包含refund_success_time的就为微信退款成功后的到账时间
				if(key.contains("refund_success_time")){
					String time = String.valueOf(entry.getValue());
					try {
						SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
						refundSuccessTime = sdf.parse(time).getTime();
						reOrder.setRefundSuccessTime(refundSuccessTime);
					} catch (Exception e) {
						e.printStackTrace();
					}
					
				}
			}
			ktjRefundOrderDao.updateTemplateById(reOrder);
			return RespWrapper.makeSuccessResp("");
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器错误！");
		}
	}

	private Map<String, Object> queryRefundMap(Map<String, Object> data) {
		Map<String,Object> map = new HashMap<String, Object>();
		map.put("appid", ProjectConstant.appid);
		map.put("mch_id", ProjectConstant.merchantId);
		map.put("sub_appid", data.get("miniProgramSubAppid"));
		map.put("sub_mch_id",  data.get("subMchid"));
		map.put("nonce_str", UUID.randomUUID().toString().replaceAll("-", "").toUpperCase());
		map.put("refund_id", data.get("refundId"));
		map.put("sign", WeChatUtil.createSign(map, ProjectConstant.api));
		return map;
	}
}
