package com.wechat.service.sepAccount.impl;

import com.wechat.common.entity.order.KtjJsapiOrder;
import com.wechat.common.enums.RegisterStatus;
import com.wechat.common.enums.SepAccountEnums;
import com.wechat.common.parameter.ProjectConstant;
import com.wechat.common.response.RespWrapper;
import com.wechat.common.util.ClientCustomSSL;
import com.wechat.common.util.StringUtil;
import com.wechat.common.util.WeChatUtil;
import com.wechat.common.util.WxXmlParser;
import com.wechat.dao.order.KtjJsapiOrderDao;
import com.wechat.dao.sepAccount.SepRecordDao;
import com.wechat.dao.sepAccount.SepUserDao;
import com.wechat.dao.sepAccount.SepUserRecordDao;
import com.wechat.entity.sepAccount.KtjSepRecord;
import com.wechat.entity.sepAccount.KtjSepUser;
import com.wechat.entity.sepAccount.KtjSepUserRecord;
import com.wechat.entity.sepAccount.help.SepReceiverHelp;
import com.wechat.entity.sepAccount.help.SepUserHandleHelp;
import com.wechat.entity.sepAccount.help.UserSepReceiver;
import com.wechat.service.sepAccount.ISepAccountService;
import org.apache.http.client.utils.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class SepAccountServiceImpl implements ISepAccountService, DisposableBean {
    // 不传 使用默认的分组
    private static final String DEFAULT_GROUP_NAME = "KTJ_SEP_GROUP";
    @Autowired
    private SepUserDao sepUserDao;
    @Autowired
    private KtjJsapiOrderDao ktjJsapiOrderDao;
    @Autowired
    private SepUserRecordDao sepUserRecordDao;
    @Autowired
    private SepRecordDao sepRecordDao;
    // 删除分账
    private static final String DEL_SEP_ACCOUNT_USER = "{\"type\":\"%s\",\"account\":\"%s\"}";
    // 添加分账
    private static final String ADD_WX_ACCOUNT_USER = "{\"type\":\"%s\",\"account\":\"%s\",\"name\":\"%s\",\"relation_type\":\"%s\"}";
    private static Logger logger = LoggerFactory.getLogger(SepAccountServiceImpl.class);

    @Override
    public RespWrapper<Boolean> addSepReceive(UserSepReceiver userSepReceiver) {
        // 1、校验该商户是否已经添加分账功能
        RespWrapper<Boolean> checkSepFun = checkSepFunction(userSepReceiver.getSubMchId());
        if(!checkSepFun.isSuccess()) return checkSepFun;
        // 2、查询所有已经添加的分账用户
        KtjSepUser user = new KtjSepUser();
        user.setSubMchId(userSepReceiver.getSubMchId());
        List<KtjSepUser> template = sepUserDao.template(user);
        String groupName = userSepReceiver.getGroupName();
        if(StringUtil.isEmpty(groupName)) groupName = DEFAULT_GROUP_NAME;
        // 获取当前分组的数据
        List<KtjSepUser> groupSepUserMap = findGroupSepUserMap(template,groupName);
        // 去掉当前分组的账号引用
        Map<String,KtjSepUser> accountMap = findWxAccountMap(template,groupName);
        template.clear();
        // 需要保证并且请求微信
        List<KtjSepUser> saveDBAndWx = new ArrayList<>(8);
        // 只需保证到数据库
        List<KtjSepUser> saveDB = new ArrayList<>(8);
        // 删除数据库并删除微信
        List<KtjSepUser> delGroupUser = new ArrayList<>(0);
        if(groupSepUserMap.isEmpty()){
            handleSepUserData(saveDBAndWx,saveDB,accountMap,userSepReceiver,groupName);
        }else {
            /**
             * 如果分组不为空
             * 1、新增的数据在分组中，则更新分组内对应的数据
             * 2、不在分组，并且在其他的分组中，需要新数据库
             * 3、不在分组，并且不在其他分组中，需新增数据库，并请求微信新增
             */
            // 需要删除的数据为
            Map<String,SepReceiverHelp> tmpGroupName = new HashMap<>();
            userSepReceiver.getSepUserList().forEach(s->{
                tmpGroupName.put(s.getWxAccount(),s);
            });
            // 其他地方没有引用、或者没有新数据引用
            delGroupUser = groupSepUserMap.stream().filter(s -> {
                return !(tmpGroupName.containsKey(s.getAccount()) || accountMap.containsKey(s.getAccount()));
            }).collect(Collectors.toList());
            groupSepUserMap.forEach(s->{
                accountMap.put(s.getAccount(),s);
            });
            groupSepUserMap.clear();
            tmpGroupName.clear();
            handleSepUserData(saveDBAndWx,saveDB,accountMap,userSepReceiver,groupName);
        }
        // 3、处理分账逻辑
        if(!saveDB.isEmpty())
            sepUserDao.insertBatch(saveDB);
        if(!saveDBAndWx.isEmpty()){
            for(KtjSepUser ktjSepUser:saveDBAndWx){
                RespWrapper<Map<String, Object>> ck = addReceiver(ktjSepUser, userSepReceiver.getSubMchId());
                if(!ck.isSuccess()) return RespWrapper.makeFailResp(ck.getMsg());
            }
            sepUserDao.insertBatch(saveDBAndWx);
        }
        if(!delGroupUser.isEmpty()){
            for(KtjSepUser ktjSepUser:saveDBAndWx){
                RespWrapper<Map<String, Object>> ck = delReceiver(ktjSepUser, userSepReceiver.getSubMchId());
                if(!ck.isSuccess()) return RespWrapper.makeFailResp(ck.getMsg());
                sepUserDao.deleteById(ktjSepUser.getId());
            }
        }
        // help gc
        saveDB.clear();
        saveDBAndWx.clear();
        delGroupUser.clear();
        return RespWrapper.makeSuccessResp(true);
    }

    @Override
    public RespWrapper<List<KtjSepUser>> findSepUserList(SepUserHandleHelp help) {
        return RespWrapper.makeSuccessResp(doFindSepUserList(help));
    }
    private List<KtjSepUser> doFindSepUserList(SepUserHandleHelp help){
        String groupName = help.getGroupName();
        if(StringUtil.isEmpty(help.getGroupName())) groupName = DEFAULT_GROUP_NAME;
        KtjSepUser user = new KtjSepUser();
        user.setSubMchId(help.getSubMchId());
        user.setGroupName(groupName);
        List<KtjSepUser> template = sepUserDao.template(user);
        return template;
    }
    private static final String SEP_DESC = "康体佳—分账";
    @Override
    public RespWrapper<String> sepAccountHandle(SepUserHandleHelp handleHelp) {
        // 1、校验该商户是否已经添加分账功能
        RespWrapper<Boolean> checkSepFun = checkSepFunction(handleHelp.getSubMchId());
        if(!checkSepFun.isSuccess()) return RespWrapper.makeFailResp(checkSepFun.getMsg());
        List<KtjSepUser> userList = doFindSepUserList(handleHelp);
        if(userList.isEmpty()) return RespWrapper.makeFailResp("请检查是否设置了分账人");
        // 查询是否有该笔订单
        RespWrapper<KtjJsapiOrder> data = findOrder(handleHelp);
        if(!data.isSuccess()) return RespWrapper.makeFailResp(data.getMsg());
        KtjJsapiOrder order = data.getData();
        Long totalFee = order.getTotalFee();//分账金额
        if(totalFee == null || totalFee < 0)
            return RespWrapper.makeFailResp("订单金额错误");
        // 判断该用户已经分账啦
        KtjSepRecord record = new KtjSepRecord();
        record.setOrderId(order.getId().intValue());
        if (sepRecordDao.templateCount(record) > 0) {
            return RespWrapper.makeFailResp("该订单已经分账啦,无需在调用分账");
        }
        List<KtjSepUserRecord> addUserList = new ArrayList<>();
        StringBuilder sb = new StringBuilder("[");
        boolean isOne = true;
        long startTime = System.currentTimeMillis();
        int amount = 0;
        for(KtjSepUser user:userList){
            Float aFloat = Float.valueOf(user.getSepRate());
            int tmpAmount = (int) (aFloat.floatValue()*totalFee);
            KtjSepUserRecord rd = new KtjSepUserRecord();
            rd.setNickName(user.getName());
            rd.setGroupName(user.getGroupName());
            rd.setCreateTime(startTime);
            rd.setWxAccount(user.getAccount());
            rd.setAmount(tmpAmount);
            rd.setWxAccount(user.getAccount());
            addUserList.add(rd);
            amount += tmpAmount;
            if(tmpAmount > 0){
                String desc = String.format(SEP_RECEIVERS, user.getType(), user.getAccount(), tmpAmount, SEP_DESC);
                if(isOne){
                    isOne = false;
                }else sb.append(",");
                sb.append(desc);
            }
        }
        String outOrderNo = findSepOutOrderNo();
        record.setOutOrderNo(outOrderNo);
        record.setCreateTime(startTime);
        record.setStatus("ACCEPTED");
        record.setAmount(amount);//分账
        record.setTransactionId(handleHelp.getTransactionId());
        record.setSep(false);
        if(isOne){
            // 说明什么都没有分账
            record.setStatus("FINISHED");
            record.setSep(true);
            sepRecordDao.insert(record,true);
        }else {
            sb.append("]");
            RespWrapper<Map<String, Object>> ck = handleSepAccount(handleHelp.getSubMchId(), handleHelp.getTransactionId(), outOrderNo, sb.toString());
            if (!ck.isSuccess()) {
                return RespWrapper.makeFailResp(ck.getMsg());
            }
            String order_id = (String) ck.getData().get("order_id");
            record.setWxOrderNo(order_id);
            sepRecordDao.insert(record,true);
        }
        addUserList.forEach(s->{
            s.setSepRecordId(record.getId());
        });
        sepUserRecordDao.insertBatch(addUserList);
        return RespWrapper.makeSuccessResp(outOrderNo);
    }

    @Override
    public RespWrapper<Boolean> findSepAccountResult(String outOrderNo) {
        KtjSepRecord record = new KtjSepRecord();
        record.setOutOrderNo(outOrderNo);
        record = sepRecordDao.templateOne(record);
        if (record == null) return RespWrapper.makeFailResp("订单编号错误");
        if("FINISHED".equals(record.getStatus()))
            return RespWrapper.makeSuccessResp(true);
        Map<String, Object> condition = new HashMap<>();
        condition.put("sub_mch_id", record.getSubMchId());//sub_mchId
        condition.put("transaction_id", record.getTransactionId());
        condition.put("out_order_no", outOrderNo);
        RespWrapper<Map<String, Object>> ck = doWxRequest(SepAccountEnums.查询分账结果, condition, false);
        if (!ck.isSuccess()) return RespWrapper.makeFailResp(ck.getMsg());
        String status = (String) ck.getData().get("status");
        switch (status) {
            case "ACCEPTED":
                return RespWrapper.makeFailResp("受理成功");
            case "PROCESSING":
                return RespWrapper.makeFailResp("处理中");
            case "FINISHED":
                return RespWrapper.makeSuccessResp(true);
            case "CLOSED":
                return RespWrapper.makeFailResp("处理失败，已关单");
        }
        return RespWrapper.makeFailResp("未知原因：" + status);
    }

    @Override
    public RespWrapper<Boolean> noSepAccountHandle(SepUserHandleHelp handleHelp) {
        // 1、校验该商户是否已经添加分账功能
        RespWrapper<Boolean> checkSepFun = checkSepFunction(handleHelp.getSubMchId());
        if(!checkSepFun.isSuccess()) return RespWrapper.makeFailResp(checkSepFun.getMsg());
        // 查询是否有该笔订单
        RespWrapper<KtjJsapiOrder> data = findOrder(handleHelp);
        if(!data.isSuccess()) return RespWrapper.makeFailResp(data.getMsg());
        KtjSepRecord record = new KtjSepRecord();
        String outOrderNo = findSepOutOrderNo("NOSEP");
        record.setOutOrderNo(outOrderNo);
        record.setTransactionId(handleHelp.getTransactionId());
        record.setSep(true);
        record.setAmount(0);
        record.setCreateTime(System.currentTimeMillis());
        record.setSubMchId(handleHelp.getSubMchId());
        record.setOrderId(data.getData().getId().intValue());
        record.setStatus("FINISHED");
        Map<String,Object> condition = new HashMap<>();
        condition.put("sub_mch_id", handleHelp.getSubMchId());//sub_mchId
        condition.put("appid",ProjectConstant.appid);// appid
        condition.put("transaction_id",handleHelp.getTransactionId());
        condition.put("sign_type","HMAC-SHA256");
        condition.put("out_order_no",outOrderNo);
        condition.put("description","分账完成");
        RespWrapper<Map<String, Object>> ck = doWxRequest(SepAccountEnums.完结分账, condition, true);
        if(!ck.isSuccess()) return RespWrapper.makeFailResp(ck.getMsg());
        String order_id = (String) ck.getData().get("order_id");
        record.setWxOrderNo(order_id);
        sepRecordDao.insert(record);
        return RespWrapper.makeSuccessResp(true);
    }

    @Override
    public void doFindSepAccountResult() {
        KtjSepRecord record = new KtjSepRecord();
        record.setSep(false);
        sepRecordDao.template(record).forEach(s->{
            Map<String, Object> condition = new HashMap<>();
            condition.put("sub_mch_id", record.getSubMchId());//sub_mchId
            condition.put("transaction_id", record.getTransactionId());
            condition.put("out_order_no", record.getOutOrderNo());
            RespWrapper<Map<String, Object>> ck = doWxRequest(SepAccountEnums.查询分账结果, condition, false);
            if (ck.isSuccess()) {
                // 查询成功
                if ("FINISHED".equals(ck.getData().get("status"))) {
                    // 这里需要更新数据
                    record.setSep(true);
                    record.setStatus("FINISHED");
                    sepRecordDao.updateTemplateById(record);
                }
            }
        });
    }



    private RespWrapper<KtjJsapiOrder> findOrder(SepUserHandleHelp handleHelp){
        KtjJsapiOrder order = new KtjJsapiOrder();
        order.setTransactionId(handleHelp.getTransactionId());//查询
        order = ktjJsapiOrderDao.templateOne(order);
        if(order == null) return RespWrapper.makeFailResp("微信支付订单号错误");
        if(!handleHelp.getSubMchId().equals(order.getSubMchid()))
            return RespWrapper.makeFailResp("请检查特约商户账号或微信支付订单号");
        if(order.getOrderStatus() == 0)
            return RespWrapper.makeFailResp("该订单还没有支付成功，无法分账");
        return RespWrapper.makeSuccessResp(order);
    }
    private static final String SEP_RECEIVERS = "{\"type\": \"%s\",\"account\":\"%s\",\"amount\":%s,\"description\": \"%s\"}";
    private RespWrapper<Map<String,Object>> handleSepAccount(String subMchId,String transactionId,String outOrderNo,String receivers){
        Map<String,Object> condition = new HashMap<>();
        condition.put("sub_mch_id", subMchId);//sub_mchId
        condition.put("appid",ProjectConstant.appid);// appid
        condition.put("transaction_id",transactionId);
        condition.put("out_order_no",outOrderNo);
        condition.put("receivers", receivers);
        return doWxRequest(SepAccountEnums.单次分账,condition,true);
    }


    private RespWrapper<Map<String, Object>> delReceiver(KtjSepUser ktjSepUser, String subMchId) {
        Map<String,Object> condition = new HashMap<>();
        condition.put("sub_mch_id", subMchId);//sub_mchId
        condition.put("appid",ProjectConstant.appid);// appid
        condition.put("receiver",String.format(DEL_SEP_ACCOUNT_USER,ktjSepUser.getType(),ktjSepUser.getAccount()));
        return doWxRequest(SepAccountEnums.删除分账接收方, condition, false);
    }

    private RespWrapper<Map<String, Object>> addReceiver(KtjSepUser user,String subMchId){
        Map<String,Object> condition = new HashMap<>();
        condition.put("sub_mch_id", subMchId);//sub_mchId
        condition.put("appid",ProjectConstant.appid);// appid
        condition.put("receiver",String.format(ADD_WX_ACCOUNT_USER,user.getType(),user.getAccount(),user.getName(),user.getRelationType()));
        return doWxRequest(SepAccountEnums.添加分账接收方, condition, false);
    }
    /**
     * 如果分组为空，则不需向微信删除分账人
     * 1、如果在数据库存在数据--> 在数据库新增一条记录
     * 2、如果没有在数据库--> 增加数据库的，并向微信请求新增
     */
    private void handleSepUserData(List<KtjSepUser> saveDBAndWx,List<KtjSepUser> saveDB,Map<String,KtjSepUser> accountMap,UserSepReceiver userSepReceiver,String groupName){
        for(SepReceiverHelp hr:userSepReceiver.getSepUserList()){
            KtjSepUser ktjSepUser = accountMap.get(hr.getWxAccount());
            // 第2种情况
            if(ktjSepUser == null){
                saveDBAndWx.add(initSepUser(hr,groupName,userSepReceiver.getSubMchId()));
            }else {
                // 第1种情况
                ktjSepUser.setGroupName(groupName);
                ktjSepUser.setSepRate(hr.getSepRate());
                ktjSepUser.setId(null);
                saveDB.add(ktjSepUser);
            }
        }
    }


    private KtjSepUser initSepUser(SepReceiverHelp hr,String groupName,String subMchId){
        KtjSepUser kt = new KtjSepUser();
        kt.setName(hr.getWxAccount());
        kt.setSepRate(hr.getSepRate());
        kt.setGroupName(groupName);
        kt.setName(hr.getNickName());
        kt.setType("PERSONAL_WECHATID");
        kt.setSubMchId(subMchId);
        kt.setRelationType("DISTRIBUTOR");
        return kt;
    }

    // 获取其他分组的引用
    private Map<String, KtjSepUser> findWxAccountMap(List<KtjSepUser> template,String groupName) {
        Map<String, KtjSepUser> map  = new HashMap<>();
        template.forEach(s->{
            if(!groupName.equals(s.getGroupName()))
                map.put(s.getAccount(),s);
        });
        return map;
    }


    private List<KtjSepUser> findGroupSepUserMap(List<KtjSepUser> template,String groupName) {
        return template.stream().filter(s->s.getGroupName().equals(groupName)).collect(Collectors.toList());
    }


    private RespWrapper<Boolean> checkSepFunction(String subMchId){
        Integer registerStatus = sepUserDao.findMerchantRegisterStatus(subMchId);
        if(registerStatus == null) return RespWrapper.makeFailResp("该商户还没有入驻,请先入驻");
        RegisterStatus status = RegisterStatus.findRegisterStatus(registerStatus);
        if(status == null) return RespWrapper.makeFailResp("该商户还没有入驻,请先入驻");
        if(status != RegisterStatus.APPLYMENT_STATE_FINISHED)
            return RespWrapper.makeFailResp(status.getStatus());
        return RespWrapper.makeSuccessResp(true);
    }


    /**
     * 包括商户号，签名类型、随机数
     * @param enums
     * @param condition
     * @param isSSL
     * @return
     */
    private RespWrapper<Map<String,Object>> doWxRequest(SepAccountEnums enums, Map<String,Object> condition, boolean isSSL){
        Assert.notNull(condition,"微信请求参数不能为空");
        condition.put("mch_id", ProjectConstant.merchantId);//商户id
        condition.put("sign_type","HMAC-SHA256");
        condition.put("nonce_str",ProjectConstant.findNonceStr());
        condition.put("sign",WeChatUtil.sha256_HMAC(condition, ProjectConstant.api));
        String xml = WeChatUtil.mapFormatXML(condition);
        try {
            String xmlStr;
            if(isSSL){
                xmlStr = ClientCustomSSL.doCertWX(enums.getUrl(), xml);
            }else xmlStr = WeChatUtil.postWX(enums.getUrl(), xml);
            Map<String,Object> resultMap = WxXmlParser.parseXML(xmlStr);   		//解析微信响应的XML结果
            if(checkNoticeSign(resultMap)){
                String return_code = (String) resultMap.get("return_code");
                if("SUCCESS".equals(return_code)){
                    String result_code = (String) resultMap.get("result_code");
                    if("SUCCESS".equals(result_code)){
                        return RespWrapper.makeSuccessResp(resultMap);
                    }
                    return RespWrapper.makeFailResp("err_code:"+resultMap.get("err_code")+",err_code_des:"+resultMap.get("err_code_des"));
                }
                return RespWrapper.makeFailResp((String) resultMap.get("return_msg"));
            }
            return RespWrapper.makeFailResp("校验签名错误");
        }catch (Exception e){
            logger.error("请求微信异常：{}",e);
            return RespWrapper.makeFailResp("请求异常,请稍后再试!异常："+e.getMessage());
        }
    }




    @Override
    public void destroy() throws Exception {
        ClientCustomSSL.stop();
    }




    /**
     * 校验微信响应数据
      */
    private static boolean checkNoticeSign(Map<String,Object> params){
        String sign = (String) params.get("sign");
        if(sign == null) return false;
        return sign.equals(WeChatUtil.sha256_HMAC(params, ProjectConstant.api));
    }

    private static final String SEP_OUT_ORDER_NO = "KTJ_%s_%s_%s";
    private static String findSepOutOrderNo(){
        return findSepOutOrderNo("SEP");
    }
    private static String findSepOutOrderNo(String sep){
        String s = DateUtils.formatDate(new Date(), "yyyyMMdd");
        return String.format(SEP_OUT_ORDER_NO,sep,s,System.nanoTime());
    }

}
